@@ -48,7 +48,7 @@ extern univptr_t emalloc __((size_t len));
 extern univptr_t erealloc __((univptr_t buf, size_t len));
 extern int efstat __((int fd, struct stat *stbuf));
 extern int estat __((char *path, struct stat *stbuf));
-extern long elseek __((int fd, long pos, int action));
+extern off_t elseek __((int fd, off_t pos, int action));
 extern int elink __((char *file1, char *file2));
 extern int eunlink __((char *file));
 extern int eclose __((int fd));
@@ -60,7 +60,7 @@ extern int erename __((char *from, char *to));
 #ifdef	HOST_NOT_FOUND	/* If we have  <netdb.h> included */
 /* hostent.c */
 extern void hp_init __((struct hostent *hp));
-extern void hp_setalist __((struct hostent *hp));
+extern void hp_setalist __((struct hostent *hp, void **));
 extern char **hp_getaddr __((void));
 extern char **hp_nextaddr __((void));
 extern void hp_addr_randomize __((struct hostent *hp));
@@ -72,7 +72,7 @@ extern int killprevious __((int sig, char *pidfile));
 /* linebuffer.c */
 extern void initline __((long blksize));
 extern int  getline __((FILE *fp));
-extern void repos_getline __((FILE *fp, long pos));
+extern void repos_getline __((FILE *fp, off_t pos));
 extern int  getrest __((void));
 extern long offset __((FILE *fp));
 
