@@ -136,8 +136,8 @@ extern void	       diagnostic __((/* struct rcpt *, int, int, char *,... */));
 #endif
 
 /* dnsgetrr.c */
-extern int	getrr     __((char *, int, int, int));
-extern int	getrrtype __((char *, int, int, int));
+extern int	getrr     __((char *, int, int, int, FILE *));
+extern int	getrrtype __((char *, int, int, int, FILE *));
 
 /* emptyline.c: */
 extern int	       emptyline __((char *line, int size));
@@ -195,7 +195,7 @@ extern void downgrade_headers __((struct rcpt *rp, int downgrade, FILE *verbosel
 extern int qp_to_8bit __((struct rcpt *rp));
 
 /* mime2headers.c */
-extern int headers_to_mime2 __((struct rcpt *rp, FILE *verboselog));
+extern int headers_to_mime2 __((struct rcpt *rp, char *defcharset, FILE *verboselog));
 
 
 /* writeheaders.c: */
