@@ -7,15 +7,15 @@
  * Perhaps a single interface to discovering the real system name.
  */
 
-#include <stdio.h>
 #include "hostenv.h"
-#if	defined(USE_HOSTS) || defined(USE_RESOLV)
+#include <stdio.h>
+#if	defined(HAVE_RESOLVER)
 #include <netdb.h>
 #include <netinet/in.h>
-#endif	/* USE_HOSTS */
-#ifdef	USE_UNAME
+#endif
+#ifdef	HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
-#endif	/* USE_UNAME */
+#endif
 
 
 int
@@ -23,11 +23,11 @@ getmyhostname(namebuf, len)
 	char *namebuf;
 	int len;
 {
-#if	defined(USE_HOSTS) || defined(USE_RESOLV)
+#if	defined(HAVE_RESOLVER)
 	struct hostent *hp;
 	struct in_addr hpaddr;
-#endif	/* USE_HOSTS */
-#ifdef	USE_UNAME
+#endif	/* HAVE_RESOLVER */
+#ifdef	HAVE_SYS_UTSNAME_H
 	struct utsname id;
 	extern int uname();
 
@@ -35,18 +35,18 @@ getmyhostname(namebuf, len)
 		return -1;
 	if (strlen(id.nodename) < len)
 		strcpy(namebuf, id.nodename);
-#else	/* !USE_UNAME */
+#else	/* !HAVE_SYS_UTSNAME_H */
 	extern int gethostname();
 	
 	if (gethostname(namebuf, len) < 0)
 		return -1;
 #endif	/* USE_UNAME */
-#if	defined(USE_HOSTS) || defined(USE_RESOLV)
+#if	defined(HAVE_RESOLVER)
 	if ((hp = gethostbyname(namebuf)) != NULL) {
 		if (strlen(hp->h_name) < len)
 			strcpy(namebuf, hp->h_name);
 	}
-#if 1
+#if 0
 	/* enable this code if for some reason your PTR name is primary */
 	if (hp != NULL)
 	  memcpy(&hpaddr,hp->h_addr,hp->h_length);
@@ -57,6 +57,6 @@ getmyhostname(namebuf, len)
 	} else
 		return 0; /* Hmm.. Didn't quite knack it ? */
 #endif
-#endif	/* USE_HOSTS */
+#endif	/* HAVE_RESOLVER */
 	return 0;
 }
