@@ -68,17 +68,22 @@ struct db_kind {
 	struct db_info	config;		/* default configuration information */
 } db_kinds[] = {
 { "incore",	{ NULL, NULL, 0, 0, 0, NULL, search_core, close_core,
-		  add_core, remove_core, print_core, count_core, owner_core, NULL, Nul } },
+		  add_core, remove_core, print_core, count_core, owner_core,
+		  NULL, Nul } },
 { "header",	{ NULL, NULL, 0, 0, 0, NULL, search_header, close_header,
-		  add_header, remove_header, print_header, count_header, owner_header, NULL, Nul } },
+		  add_header, remove_header, print_header, count_header,
+		  owner_header, NULL, Nul } },
 { "unordered",	{ NULL, NULL, 0, 10, 0, NULL, search_seq, close_seq,
-		  add_seq, NULL, print_seq, count_seq, owner_seq, modp_seq, Nul } },
+		  add_seq, NULL, print_seq, count_seq, owner_seq, modp_seq,
+		  Nul } },
 #ifndef	HAVE_MMAP
 { "ordered",	{ NULL, NULL, 0, 10, 0, NULL, search_bin, close_seq,
-		  NULL, NULL, print_seq, count_seq, owner_seq, modp_seq, Nul }	},
+		  NULL, NULL, print_seq, count_seq, owner_seq, modp_seq,
+		  Nul } },
 #else /* HAVE_MMAP */ /* When using MMAP(), no cache is needed for ordered.. */
 { "ordered",	{ NULL, NULL, 0, 0, 0, NULL, search_bin, close_seq,
-		  NULL, NULL, print_seq, count_seq, owner_seq, modp_seq, Nul }	},
+		  NULL, NULL, print_seq, count_seq, owner_seq, modp_seq,
+		  Nul } },
 #endif
 
 #ifdef	HAVE_HOSTS
@@ -90,7 +95,7 @@ struct db_kind {
 # define RESOLV_CONF "/etc/resolv.conf"
 #endif
 { "bind",	{ RESOLV_CONF, NULL, 0, 10, 0, NULL,
-		  search_res, NULL, NULL, NULL, NULL, NULL, NULL, NULL, Nul }	},
+		  search_res, NULL, NULL, NULL, NULL, NULL, NULL, NULL, Nul }},
 #endif	/* HAVE_RESOLV */
 #ifdef	HAVE_NDBM_H
 { "ndbm",	{ NULL, NULL, 0, 10, 0, NULL, search_ndbm, close_ndbm,
@@ -107,9 +112,14 @@ struct db_kind {
 		  add_dbm, remove_dbm, print_dbm, count_dbm, owner_dbm,
 		  NULL, Nul } },
 #endif	/* HAVE_DBM */
+#ifdef	HAVE_DB_H
+{ "btree",	{ NULL, NULL, 0, 10, 0, NULL, search_btree, close_btree,
+		  add_btree, remove_btree, print_btree, count_btree,
+		  owner_btree, NULL, Nul } },
+#endif	/* HAVE_DB_H */
 #ifdef	HAVE_YP
 { "yp",		{ NULL, NULL, 0, 10, 0, NULL, search_yp, NULL,
-		  NULL, NULL, print_yp, NULL, owner_yp, NULL, Nul }	},
+		  NULL, NULL, print_yp, NULL, owner_yp, NULL, Nul } },
 #endif	/* HAVE_YP */
 { 0 }	/* proto_config is initialized from this entry */
 };
@@ -668,7 +678,7 @@ db(dbname, key)
 	j = 0;
 	k = 0;
 	if (dbip->cache_size > 0) {
-		for (i=j=0; i < dbip->cache_size && CACHE(i,key) != NULL; ++i) {
+		for (i=j=0; i < dbip->cache_size && CACHE(i,key) != NULL; ++i){
 			if (CACHE(i,expiry) > 0 && CACHE(i,expiry) < now) {
 				if (D_db)
 					fprintf(stderr,
@@ -975,6 +985,11 @@ find_domain(lookupfn, sip)
 			return l;
 	}
 #endif
+	/* Still failed ?  Try to look for "." */
+	sip->key = ".";
+	if ((l = (*lookupfn)(sip)) != NULL)
+		return l;
+	
 	return NULL;
 }
 
