@@ -650,7 +650,7 @@ rd_doit(filename, dirs)
 	}
 
 #ifdef	MALLOC_TRACE
-mal_contents(stdout);
+	mal_contents(stdout);
 #endif
 
 	gensym = 1;
@@ -663,7 +663,7 @@ mal_contents(stdout);
 	setlevel(MEM_SHCMD,sh_memlevel);
 
 #ifdef MALLOC_TRACE
-mal_contents(stdout);
+	mal_contents(stdout);
 #endif
 
 	return 1;
@@ -1512,10 +1512,10 @@ run_listexpand(avl, il)
 		s = buf;
 		while (*s) {
 		  u_char c = *s;
-		  if ('!' <= c && c <= '~' && c != '+' && c != '=')
+		  if ('!' <= c && c <= '~' && c != '+' && c != '=') {
 		    if (s2 < se)
 		      *s2++ = c;
-		  else if (s2 < se) {
+		  } else if (s2 < se) {
 		    sprintf(s2,"+%02X",c);
 		    s2 += 3;
 		  }
