@@ -199,7 +199,10 @@ search_res(sip)
 	 * The default (4) will cause the router to fall
 	 * hopelessly behind in the face of broken DNS data.
 	 */
-	_res.retry   = 2;
+	/* _res.retry   = 2; */
+	/* Lets not, but lets not use DNS at router, lets use it only
+	   at SMTP channels */
+
 	if (sip->subtype == NULL || *(sip->subtype) == '\0') {
 	  fprintf(stderr, "search_res: missing subtype to BIND query!\n");
 	  return NULL;
