@@ -43,7 +43,9 @@ struct headerinfo mandatory_hdrs[] = {
 
 struct headerinfo optional_hdrs[] = {
 { "return-receipt-to",	AddressList,	Sender,		normal		},
-{ "return-path",	Mailbox,	Sender,		normal		},
+#if 0
+{ "return-path",	AMailboxList,	nilUserType,	normal		},
+#endif
 { "date",		DateTime,	nilUserType,	normal		},
 { "resent-date",	DateTime,	nilUserType,	Resent		},
 { "encrypted",		Encrypted,	nilUserType,	normal		},
