@@ -55,7 +55,7 @@ char	*mailshare;
 char	*myhostname = 0;
 extern char *postoffice; /* at mail_open() */
 char	*pidfile = PID_ROUTER;
-char	*log;
+char	*logfn;
 time_t	now;
 
 extern memtypes stickymem;
@@ -88,7 +88,7 @@ main(argc, argv)
 	else
 		++progname;
 	origoptind = optind;	/* needed for reuse of getopt() */
-	log = config = NULL;
+	logfn = config = NULL;
 	errflg = daemonflg = killflg = interactiveflg = version = 0;
 	tac = 0;
 	nrouters = 1;
@@ -147,7 +147,7 @@ main(argc, argv)
 			nosyslog = 0;
 			break;
 		case 'L':	/* override default log file */
-			log = optarg;
+			logfn = optarg;
 			break;
 		case 'P':	/* override default postoffice */
 			postoffice = optarg;
@@ -191,18 +191,18 @@ main(argc, argv)
 
 	c = optind;	/* save optind since builtins can interfere with it */
 
-	if (daemonflg && log == NULL) {
+	if (daemonflg && logfn == NULL) {
 		if ((cp = getzenv("LOGDIR")) != NULL)
 			logdir = cp;
-		log = smalloc(MEM_PERM, 2 + (u_int)(strlen(logdir)
+		logfn = smalloc(MEM_PERM, 2 + (u_int)(strlen(logdir)
 					  + strlen(progname)));
-		sprintf(log, "%s/%s", logdir, progname);
+		sprintf(logfn, "%s/%s", logdir, progname);
 	}
 	/* setvbuf(stdout, (char *)NULL, _IOLBF, 0);
 	   setvbuf(stderr, (char *)NULL, _IOLBF, 0); */
 	setvbuf(stdout, (char *)NULL, _IONBF, 0);
 	setvbuf(stderr, (char *)NULL, _IONBF, 0);
-	if (log != NULL) {
+	if (logfn != NULL) {
 		/* loginit is a signal handler, so can't pass log */
 		if (loginit() < 0)	/* do setlinebuf() there */
 			die(1, "log initialization failure");
