@@ -34,3 +34,7 @@ int	sweepinterval = 10;
 
 /* At most 10 new childs per second -- The 'R'-option can be used to increase this */
 int	forkrate_limit = 10;
+
+/* Set if you don't want to return full content with error report! */
+int	never_full_content = 0;
+
