@@ -17,7 +17,7 @@
 #include "mail.h"
 
 static void celink __((struct config_entry *, struct config_entry **, struct config_entry **));
-static int readtoken __((FILE *fp, char *buf, int buflen));
+static int readtoken __((FILE *fp, char *buf, int buflen, int *linenump));
 static u_int parse_intvl __((char *string));
 
 static int rc_command  __((char *key, char *arg, struct config_entry *ce));
@@ -228,6 +228,7 @@ readconfig(file)
 	struct vertex v;
 	struct threadgroup tg;
 	FILE *fp;
+	int linenum = 0;
 
 	ce = head = tail = NULL;
 	errflag = 0;
@@ -237,7 +238,7 @@ readconfig(file)
 		  progname, file, strerror(errno));
 	  return NULL;
 	}
-	while ((n = readtoken(fp, line, sizeof line)) != -1) {
+	while ((n = readtoken(fp, line, sizeof line, &linenum)) != -1) {
 	  if (verbose)
 	    printf("read '%s' %d\n",  line, n);
 	  if (n == 1) {
@@ -269,16 +270,26 @@ readconfig(file)
 	    } else
 	      defaultconfigentry(ce,default_entry);
 	  } else if (ce != NULL) {
+	    a = NULL;
 	    if ((cp = strchr(line, '=')) != NULL) {
+	      char *p = cp-1;
 	      *cp = '\0';
+	      while (p >= line && (*p == ' ' || *p == '\t'))
+		*p-- = '\0';
 	      a = cp+1;
+	      while (*a == ' ' || *a == '\t') ++a;
 	      if (*a == '"') {
 		++a;
-		cp = strrchr(a, '"');
-		*cp = '\0';
+		cp = a;
+		while (*cp && *cp != '"') {
+		  if (*cp == '\\' && cp[1] != 0)
+		    ++cp;
+		  ++cp;
+		}
+		if (*cp)
+		  *cp = '\0';
 	      }
-	    } else
-	      a = NULL;
+	    }
 	    ce->mark = 0;
 	    for (rckp = &rckeys[0]; rckp->name != NULL ; ++rckp)
 	      if (cistrcmp(rckp->name, line) == 0) {
@@ -287,8 +298,8 @@ readconfig(file)
 	      }
 	    if (rckp->name == NULL) {
 	      fprintf(stderr,
-		      "%s: unknown keyword %s in %s\n",
-		      progname, line, file);
+		      "%s: unknown keyword %s in %s:%d\n",
+		      progname, line, file, linenum);
 	      ++errflag;
 	    }
 	  } else {
@@ -311,32 +322,50 @@ readconfig(file)
 }
 
 static int
-readtoken(fp, buf, buflen)
+readtoken(fp, buf, buflen, linenump)
 	FILE *fp;
 	char *buf;
-	int buflen;
+	int buflen, *linenump;
 {
 	static char line[BUFSIZ];
 	static char *lp = NULL;
 	char *elp;
 	int rv;
 
+redo_readtoken:
 	if (lp == NULL) {
 	  if (fgets(line, sizeof line, fp) == NULL)
 	    return -1;
+	  *linenump += 1;
 	  lp = line;
 	}
+	/* Skip initial white-space */
 	while (*lp != '\0' && isspace(*lp))
 	  ++lp;
+	/* Now it is one of: a token, a comment start, or end of line */
 	if (*lp == '\0' || *lp == '#') {
+	  /* Comment/EOL */
 	  lp = NULL;
-	  return readtoken(fp, buf, buflen);
+	  goto redo_readtoken;
 	}
-	for (elp = lp; *elp != '\0'; ++elp) {
-	  if (isspace(*elp))
-	    break;
-	  if (*elp == '=' && *(elp+1) == '"') {
-	    elp += 2;
+	/* Now we scan for the token + possible value */
+	elp = lp;
+	while (*elp && !isspace(*elp) && *elp != '=' && *elp != '#')
+	  ++elp;
+	if (isspace(*elp)) {
+	  /* Allow spaces after the token and before '=' */
+	  char *p = elp;
+	  while (*p && isspace(*p)) ++p;
+	  if (*p == '=')
+	    elp = p;
+	}
+	/* Value indicator ? */
+	if (*elp == '=') {
+	  /* Allow spaces between '=', and value */
+	  ++elp;
+	  while (*elp && isspace(*elp)) ++elp;
+	  if (*elp == '"') {
+	    ++elp;
 	    while (*elp != '"' && *elp != '\0') {
 	      if (*elp == '\\' && *(elp+1) == '\n') {
 		if (fgets(elp, sizeof line - (elp - line), fp) == NULL) {
@@ -354,6 +383,10 @@ readtoken(fp, buf, buflen)
 		      progname, line);
 	      return -1;
 	    }
+	    ++elp;
+	  } else {
+	    while (*elp && !isspace(*elp) && *elp != '"')
+	      ++elp;
 	  }
 	}
 	strncpy(buf, lp, elp-lp);
