@@ -95,6 +95,13 @@ int	hashlevels = 0;		/* How many levels of hashes are supported
 				   at the transport directory ?  (To speed
 				   up the directory processing, file opens,
 				   etc.. ) */
+char	*procselect = NULL;	/* Non-null defines  channel/host specifier
+				   that is ALLOWED TO RUN, specifying this
+				   (by means of '-P chan/host' -option) will
+				   prevent running anything else, and also
+				   prevent running error processing, or
+				   job-specifier deletions. */
+char	*procselhost = NULL;	/* Just spliced out 'host'-part of the above */
 extern int forkrate_limit;	/* How many forks per second ? */
 
 static int vtxprep_skip      = 0;
@@ -273,7 +280,7 @@ main(argc, argv)
 	dlyverbose = 0;
 	syncstart = 0;
 	verbose = errflg = version = 0;
-	while ((c = getopt(argc, argv, "divE:f:Fl:HL:N:P:q:QR:SVW")) != EOF)
+	while ((c = getopt(argc, argv, "divE:f:Fl:HL:N:p:P:q:QR:SVW")) != EOF)
 		switch (c) {
 		case 'f':	/* override default config file */
 			config = optarg;
@@ -304,6 +311,16 @@ main(argc, argv)
 			if ((transportmaxnofiles = atoi(optarg)) < 10)
 				transportmaxnofiles = TRANSPORTMAXNOFILES;
 			break;
+		case 'p':
+			procselect = optarg;
+			procselhost = strchr(procselect,'/');
+			if (procselhost)
+			  *procselhost++ = 0;
+			else {
+			  fprintf(stderr,"scheduler: -p parameter is of form: channel/host\n");
+			  exit(64);
+			}
+			break;
 		case 'P':	/* override default postoffice */
 			postoffice = optarg;
 			break;
@@ -1218,6 +1235,7 @@ static struct ctlfile *vtxprep(cfp, file)
 	int offspc;
 	long ino;
 	int prevrcpt = -1;
+	int is_turnme = 0;
 
 	char fpath[128], path[128], path2[128];
 
@@ -1420,12 +1438,17 @@ static struct ctlfile *vtxprep(cfp, file)
 	      printf("TURNME: %s\n",cp+2);
 	      strlower(cp+2);
 	      turnme(cp+2);
+	      is_turnme = 1;
 	      break;
 	    }
 	  }
 	}
 	close(cfp->fd);	/* closes the fd opened in schedule() */
 
+	if (is_turnme && cfp->mid == NULL)
+	  unlink(file);
+
+
 	if (fpath[0] >= 'A') {
 	  /* Prefixed with a subdir path */
 	  int hash = 0;
@@ -1442,7 +1465,7 @@ static struct ctlfile *vtxprep(cfp, file)
 
 	} else { /* Not at subdirs */
 
-	  if (hashlevels > 0) {
+	  if (cfp->mid && hashlevels > 0) {
 	    /* We have a desire to use subdirs, now the magic of hashing
 	       into subdirs...  inode number ? */
 
