@@ -30,6 +30,7 @@ int idleprocs = 0;
 extern int global_wrkcnt;
 extern int syncstart;
 extern time_t now;
+extern char *procselect, *procselhost;
 
 struct threadgroup *
 create_threadgroup(cep, wc, wh, withhost, ce_fillin)
@@ -687,11 +688,19 @@ struct thread *thr;
 
 
 	if (syncstart) return 0;
+	if (procselect) {
+	  if (*procselect != '*' &&
+	      strcmp(procselect,ch->name) != 0)
+	    return 0;
+	  if (*procselhost != '*' &&
+	      strcmp(procselhost,ho->name) != 0)
+	    return 0;
+	}
 
 	if (verbose)
 	  fprintf(stderr,"thread_start(thr=%s/%d/%s) (dt=%d, thr=0x%p)\n",
-		  vp->orig[L_CHANNEL]->name, thg->withhost,
-		  vp->orig[L_HOST]->name, (int)(thr->wakeup-now),thr);
+		  ch->name, thg->withhost, ho->name, (int)(thr->wakeup-now),
+		  thr);
 
 	if (thr->proc != NULL &&
 	    thr->proc->thread == thr) {
@@ -1446,7 +1455,7 @@ FILE *fp;
 	  fprintf(fp,"\tThreads: %4d",thg->threads);
 	  if (thg->threads != cnt)
 	    fprintf(fp,"/%d",cnt);
-	  fprintf(fp," Rcpts: %5d",jobsum);
+	  fprintf(fp," Msgs: %5d",jobsum);
 	  fprintf(fp," Procs: %3d",thg->transporters);
 	  /* 	  if (thg->transporters != procs)
 		  fprintf(fp,"/%d",procs);		*/
