@@ -663,11 +663,11 @@ time_t timeout;
 		fprintf(stderr,"** Invalid fd on a select() wrmask: %d\n",i);
 	    }
 	    fflush(stderr);
-	    abort();
+	    abort(); /* mux() select() error EINVAL or EBADF !? */
 	  }
 	  perror("select() returned unknown error ");
 	  fflush(stderr);
-	  abort();
+	  abort(); /* Select with unknown error */
 	} else if (n == 0) {
 	  /* fprintf(stderr, "abnormal 0 return from select!\n"); */
 	  /* -- just a timeout -- fast or long */
