@@ -1294,7 +1294,7 @@ int insecure;
 	  type(452,m430,(char *)NULL);
 	  clearerr(mfp);
 	} else if (sizeoptval > maxsize && maxsize > 0) {
-	  type(552,"5.3.4","The message size exceeds fixed maximum message size");
+	  type(552,"5.3.4","The message size exceeds fixed maximum of %ld chars for message size", maxsize);
 	} else if (sizeoptval > availspace) {
 	  type(452,"4.3.1","Try again later, insufficient storage available at the moment");
 	} else {
@@ -1473,7 +1473,7 @@ char *buf, *cp;
 	  type(452,m430,(char *)NULL);
 	  clearerr(mfp);
 	} else if (maxsize > 0 && sizeoptsum > maxsize) {
-	  type(552, "5.3.4", "Message size exceeds fixed maximum size for acceptable email");
+	  type(552, "5.3.4", "Message size exceeds fixed maximum size of %ld chars for acceptable email", maxsize);
 	} else if (sizeoptsum > availspace) {
 	  type(452, "4.3.1", "insufficient storage space, try again later");
 	} else if (s) {
@@ -1597,7 +1597,7 @@ smtp_data(buf,cp)
 	} else if (maxsize > 0 && filsiz > maxsize) {
 	  mail_abort(mfp);
 	  mfp = NULL;
-	  type(552, "5.3.4", "Message size exceeded fixed maximum size for acceptable email");
+	  type(552, "5.3.4", "Message size exceeded fixed maximum size of %ld chars for acceptable email", maxsize);
 	  fflush(stdout);
 	} else if (mail_close(mfp) == EOF) {
 	  type(452,m430,(char *)NULL);
@@ -1729,7 +1729,7 @@ smtp_bdata(buf,cp)
 	} else if (maxsize > 0 && tell > maxsize) {
 	  mail_abort(mfp);
 	  mfp = NULL;
-	  type(552, "5.3.4", "Message size exceeded fixed maximum size for acceptable email");
+	  type(552, "5.3.4", "Message size exceeded fixed maximum size of %ld chars for acceptable email", maxsize);
 	} else if (bdata_last) {
 	  if (fstat(FILENO(mfp), &stbuf) < 0)
 	    inum = 0;
