@@ -81,12 +81,12 @@ extern char *malloc();
 
 #ifndef __STDC__
 extern FILE *fopen();
-/* extern char *sprintf(); */
+/* extern char *sprintf();
+   extern int fprintf();
+   extern int printf(); */
 extern char *strcpy();
 extern int atoi();
-extern int fprintf();
 extern int getopt();
-extern int printf();
 extern int strcmp();
 extern int strlen();
 /* extern int tolower(); */
@@ -981,7 +981,7 @@ main(argc, argv)
 			++sslPointer;
 			break;
 		case oError:
-			Error(sslTable[sslPointer]);
+			Error((ErrorCodes)sslTable[sslPointer]);
 			++sslPointer;
 			break;
 		case oChoice:
@@ -1017,7 +1017,7 @@ main(argc, argv)
 
 		/* Symbol Table Mechanism Operations */
 		case oSetClass:
-			xSetCurrentClass(parameterValue);
+			xSetCurrentClass((SymbolClass)parameterValue);
 			break;
 		case oSetClassFromSymbolClass:
 			xSymbolClass(c);
@@ -1025,7 +1025,7 @@ main(argc, argv)
 			break;
 		case oxSetClassFromSymbolValue:
 			SymbolValue(v);
-			xSetCurrentClass(v);
+			xSetCurrentClass((SymbolClass)v);
 			break;
 		case oySetClassFromSymbolResultClass:
 			SymbolResultClass(c);
@@ -2405,7 +2405,7 @@ GenerateSymbolDefinitions()
 		c = symClass[s];
 
 		if (c == cType) {
-			GenerateClass(symValue[s], symText[s]);
+			GenerateClass((SymbolClass)symValue[s], symText[s]);
 			putc('\n', headerfp);
 		}
 	}
@@ -2582,7 +2582,7 @@ SslSyntaxError()
 	/* token, generate a dummy one.	*/
 	if (sslTable[sslPointer] >= (int)firstCompoundToken
 	    && sslTable[sslPointer] <= (int)lastCompoundToken)
-		SslGenerateCompoundInputToken(sslTable[sslPointer]);
+		SslGenerateCompoundInputToken((InputTokens)sslTable[sslPointer]);
 }
 
 
