@@ -256,17 +256,17 @@ process(dp)
 	    case EX_SOFTWARE:
 	      action = "failed";
 	      status = "5.0.0 (ZMailer internal protocol error)";
-	      diagnostics = "500 (Protocol failure inside ZMAILER! AARGH!)";
+	      diagnostics = "x-local; 500 (Protocol failure inside ZMAILER! AARGH!)";
 	      break;
 	    case EX_TEMPFAIL:
 	      action = "delayed";
 	      status = "4.4.3 (Temporary routing lookup failure)";
-	      diagnostics = "466 (Temporary routing lookup failure)";
+	      diagnostics = "x-local; 466 (Temporary routing lookup failure)";
 	      break;
 	    default:
 	      action = "failed";
 	      status = "4.4.0 (Unknown ZMailer HOLD status ** CAN'T HAPPEN)";
-	      diagnostics = "500 (Unknown HOLD return code **CAN'T HAPPEN**)";
+	      diagnostics = "x-local; 500 (Unknown HOLD return code **CAN'T HAPPEN**)";
 	      break;
 	    }
 	    notaryreport(rp->addr->user,action,status,diagnostics);
@@ -287,7 +287,7 @@ process(dp)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user,"delayed",
 			   "4.3.1 (System spool full?)",
-			   "400 (Cannot resubmit anything, out of spool space?)");
+			   "x-local; 400 (Cannot resubmit anything, out of spool space?)");
 	      diagnostic(rp, EX_TEMPFAIL, 0,
 			 "cannot resubmit anything!");
 	    }
@@ -303,29 +303,20 @@ process(dp)
 	  fprintf(mfp, "from <%s>\n", dp->senders->user);
 	if (dp->envid != NULL)
 	  fprintf(mfp, "envid %s\n", dp->envid);
-	
+	if (dp->dsnretmode)
+	  fprintf(mfp, "notaryret %s\n", dp->dsnretmode);
+
 	for (rp = dp->recipients; rp != NULL; rp = rp->next)
 	  if (rp->status == EX_OK) {
 	    fprintf(mfp, "to <%s>\n", rp->addr->user);
-	    if (rp->dsnflags != 0 || rp->orcpt != NULL) {
+	    if (rp->notify || rp->orcpt) {
 
 	      fputs("todsn",mfp);
 	      if (rp->orcpt != NULL)
 		fprintf(mfp," ORCPT=%s",rp->orcpt);
 
-	      if (rp->dsnflags & _DSN_RET_YES)
-		fputs(" RET=YES",mfp);
-	      else if (rp->dsnflags & _DSN_RET_NO)
-		fputs(" RET=NO",mfp);
-
-	      if (rp->dsnflags & _DSN_NOTIFY_SUCCESS)
-		fputs(" NOTIFY=SUCCESS",mfp);
-	      else if (rp->dsnflags & _DSN_NOTIFY_FAILURE)
-		fputs(" NOTIFY=FAILURE",mfp);
-	      else if (rp->dsnflags & _DSN_NOTIFY_ALWAYS)
-		fputs(" NOTIFY=ALWAYS",mfp);
-	      else if (rp->dsnflags & _DSN_NOTIFY_NEVER)
-		fputs(" NOTIFY=NEVER",mfp);
+	      if (rp->notify)
+		fprintf(mfp," NOTIFY=%s", rp->notify);
 
 	      putc('\n',mfp);
 	    }
@@ -353,7 +344,7 @@ process(dp)
 	  if (rp->status == EX_OK) {
 	    notaryreport(rp->addr->user,"relayed",
 			 "2.2.0 (Relayed via deferral channel)",
-			 "250 (Relayed via deferral channel)");
+			 "x-local; 250 (Relayed via deferral channel)");
 	    diagnostic(rp, code, 0, cp);
 	  }
 }
