@@ -100,20 +100,21 @@ extern int h_errno;
 extern FILE *verboselog;
 
 int
-getrr(host, hbsize, rrtype, cnamelevel)	/* getrrtype with completion */
+getrr(host, hbsize, rrtype, cnamelevel, verboselog)	/* getrrtype with completion */
 	char *host;
 	int hbsize;
 	int rrtype;
 	int cnamelevel;
+	FILE *verboselog;
 {
 	int rval;
 	char buf[BUFSIZ], **domain;
 
-	if ((rval = getrrtype(host, hbsize, rrtype, cnamelevel)) > 0)
+	if ((rval = getrrtype(host, hbsize, rrtype, cnamelevel, verboselog)) > 0)
 	  return rval;
 	for (domain = _res.dnsrch; *domain != NULL; domain++) {
 	  sprintf(buf, "%s.%s", host, *domain);
-	  if ((rval = getrrtype(buf, BUFSIZ, rrtype, cnamelevel)) > 0) {
+	  if ((rval = getrrtype(buf, BUFSIZ, rrtype, cnamelevel, verboselog)) > 0) {
 	    strncpy(host, buf, hbsize<BUFSIZ?hbsize:BUFSIZ);
 	    host[hbsize - 1] = '\0';
 	    return rval;
@@ -124,11 +125,12 @@ getrr(host, hbsize, rrtype, cnamelevel)	/* getrrtype with completion */
 
 
 int
-getrrtype(host, hbsize, rrtype, cnamelevel)
+getrrtype(host, hbsize, rrtype, cnamelevel, verboselog)
 	char *host;
 	int hbsize;
 	int rrtype;
 	int cnamelevel;
+	FILE *verboselog;
 {
 
 	HEADER *hp;
