@@ -12,7 +12,7 @@
 #include "ta.h"
 #include "sysprotos.h"
 
-extern FILE *verboselog;
+/* extern FILE *verboselog; */
 
 #ifndef	__
 # ifdef __STDC__
@@ -26,7 +26,7 @@ extern FILE *verboselog;
 extern void *emalloc();
 extern void *erealloc();
 
-#define DEFCHARSET "UNKNOWN-8BIT"
+/* #define DEFCHARSET "UNKNOWN-8BIT" */
 
 int cvtspace_copy __((struct rcpt *rp));
 
@@ -137,7 +137,8 @@ header8bit2QPnextword(line, word, len, inside, start_encode, fold, hdr_offset)
  */
 
 static int
-header8bit2QP(line, defcharset, outp, sizep, osizep, fold)
+header8bit2QP(verboselog, line, defcharset, outp, sizep, osizep, fold)
+     FILE *verboselog;
      unsigned char *line;
      const char *defcharset;
      unsigned char **outp;
@@ -197,8 +198,9 @@ header8bit2QP(line, defcharset, outp, sizep, osizep, fold)
    recommended maximum, or ... then things aren't perfect. */
 
 int
-headers_to_mime2(rp,verboselog)
+headers_to_mime2(rp,defcharset,verboselog)
 	struct rcpt *rp;
+	char *defcharset;
 	FILE *verboselog;
 {
 #ifndef USE_NEW_MIME2_CODE
@@ -304,7 +306,7 @@ if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
 	      char *outstr = emalloc(sumlen);
 	      int osize = 0;
 
-	      rc = header8bit2QP(sumstr,DEFCHARSET,&outstr,&size,&osize,1);
+	      rc = header8bit2QP(verboselog,sumstr,defcharset,&outstr,&size,&osize,1);
 	      s = outstr;
 
 	      if (outhdr == NULL)
@@ -340,7 +342,7 @@ if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
 	  char *outstr = emalloc(sumlen);
 	  int osize = 0;
 
-	  rc = header8bit2QP(sumstr,DEFCHARSET,&outstr,&size,&osize,1);
+	  rc = header8bit2QP(verboselog,sumstr,defcharset,&outstr,&size,&osize,1);
 	  s = outstr;
 
 	  size = strlen(s);
