@@ -53,7 +53,7 @@ extern void *erealloc();
 
 static int mime_received_convert __((struct rcpt *rp, char *convertstr));
 
-extern FILE *verboselog;
+/* extern FILE *verboselog; */
 
 #define istokenchar(x) (isalnum(x) || x == '-' || x == '_' || x == '$' || \
 			x == '#'   || x == '%' || x == '+')
@@ -585,6 +585,9 @@ struct rcpt *rp;
 	return 0;	/* No "Content-Conversion:" header */
 }
 
+static char *cCTE = "Content-Transfer-Encoding:";
+static char *cCT  = "Content-Type:";
+
 int
 cte_check(rp)
 struct rcpt *rp;
@@ -601,7 +604,7 @@ struct rcpt *rp;
 
 	while (*hdrs && (!mime || !cte)) {
 	  char *buf = *hdrs;
-	  if (!cte && cistrncmp(buf,"Content-Transfer-Encoding:",26)==0) {
+	  if (!cte && cistrncmp(buf,cCTE,26)==0) {
 	    buf += 26;
 	    while (*buf == ' ' || *buf == '\t') ++buf;
 	    if (*buf == '8' /* 8BIT */) cte = 8;
@@ -666,8 +669,8 @@ FILE *verboselog;
 	struct ct_data *ct;
 
 	/* Convert IN PLACE! -- if there is a need.. */
-	CT = has_header(rp,"Content-Type:");
-	CTE  = has_header(rp,"Content-Transfer-Encoding:");
+	CT = has_header(rp,cCT);
+	CTE  = has_header(rp,cCTE);
 	if (CT == NULL || CTE == NULL) return 0; /* ??? */
 
 	ct = parse_content_type(CT);
@@ -723,8 +726,8 @@ FILE *verboselog;
 	  while ((*oldmsgheader)[lines]) ++lines;
 
 	MIME = has_header(rp,"MIME-Version:");
-	CT   = has_header(rp,"Content-Type:");
-	CTE  = has_header(rp,"Content-Transfer-Encoding:");
+	CT   = has_header(rp,cCT);
+	CTE  = has_header(rp,cCTE);
 
 	if (verboselog)
 	  fprintf(verboselog,"Header conversion control code: %d\n",convertmode);
@@ -761,7 +764,7 @@ NULL };
 	    delete_header(rp,MIME);
 	  }
 	  if (CTE) {
-	    CTE  = has_header(rp,"Content-Transfer-Encoding:");
+	    CTE  = has_header(rp,cCTE);
 	    delete_header(rp,CTE);
 	  }
 
@@ -862,10 +865,11 @@ mime_received_convert(rp, convertstr)
 	  ctlfree(rp->desc,*inhdr);
 	  *inhdr = newreceived;
 
-	  if (verboselog) {
-	    fprintf(verboselog,"Rewriting 'Received:' headers.\n");
-	    fprintf(verboselog,"The new line is: '%s'\n",*inhdr);
-	  }
+	  /* if (verboselog) {
+	     fprintf(verboselog,"Rewriting 'Received:' headers.\n");
+	     fprintf(verboselog,"The new line is: '%s'\n",*inhdr);
+	     }
+	   */
 
 	  inhdr = NULL; /* quit this header munging.. */
 	}
@@ -877,32 +881,35 @@ int /* Return non-zero for success */
 qp_to_8bit(rp)
 	struct rcpt *rp;
 {
-	int ok = 0;
 	char **inhdr;
-	int cte   = 0;	/* Change C-T-E: QUOTED-PRINTABLE  to  C-T-E: 8BIT */
+	char *hdr, *p;
+	char **CTE;
 
-	if (*(rp->newmsgheadercvt) != NULL)
-	  return 1;	/* Already converted */
-	
 	if (!cvtspace_copy(rp))
 	  return 0;	/* Failed to copy ! */
 
-	if (!mime_received_convert(rp," convert rfc822-to-8bit"))
-	  return 0;	/* "Received:" conversion failed! */
-
 	inhdr = *(rp->newmsgheadercvt);
 
-	while (inhdr && *inhdr) {
-	  char *hdr = *inhdr;
-	  if (!cte && cistrncmp(hdr,
-				"Content-Transfer-Encoding:",26)==0) {
-	    strcpy(hdr,"Content-Transfer-Encoding: 8BIT");
-	    ok = 1;
-	    break; /* Done with it.. */
+	CTE = has_header(rp,cCTE);
+
+	if (!CTE) return 0; /* No C-T-E header! */
+
+	p = hdr + 26;
+	while (*p == ' ' || *p == '\t') ++p;
+	if (*p == 'Q' || *p == 'q') {
+	  if (strlen(hdr+26) >= 5)
+	    strcpy(hdr+26," 8BIT");
+	  else { /* No room ??  What junk ?? */
+	    delete_header(rp,CTE);
+	    append_header(rp,"Content-Transfer-Encoding: 8BIT");
 	  }
-	  ++inhdr;
-	}
-	return ok;
+
+	  if (!mime_received_convert(rp," convert rfc822-to-8bit"))
+	    return 0;	/* "Received:" conversion failed! */
+
+	} /* else propably already decoded */
+
+	return 1;
 }
 
 
