@@ -44,6 +44,11 @@ writeheaders(rp,fp,newline,convertmode,maxwidth)
 
 	while (*msgheaders && !ferror(fp)) {
 		int linelen = strlen(*msgheaders);
+		if (**msgheaders == '.')
+		  putc('.',fp); /* ALWAYS double-quote the begining
+				   dot -- though it should NEVER occur
+				   in the headers, but better safe than
+				   sorry.. */
 		if (fwrite(*msgheaders,1,linelen,fp) != linelen)
 			return -1;
 		hsize += linelen;
