@@ -38,7 +38,7 @@ dotlock(file)
 
 	sprintf(lockname, "%s.lock", file);
 	sprintf(temp, "%s.L%x.%lx.%x",
-		file, getpid(), (long)time((time_t *)0), gethostid());
+		file, getpid(), time(NULL), gethostid());
 	unlink(temp);
 	for (;;) {
 		if ((fd = open(temp, O_WRONLY|O_CREAT|O_EXCL, 0666)) < 0)
