@@ -28,6 +28,7 @@
  *		'N' -- NFSMBOX (uses special locking daemon at remote system)
  */
 
+#define DefCharset "UNKNOWN-8BIT"
 
 #include "hostenv.h"
 #include <stdio.h>
@@ -270,6 +271,7 @@ int   mime_boundary_len = 0;
 int mmdf_mode = 0;		/* Write out MMDF-style mail folder
 				   ("\001\001\001\001" as the separator..)  */
 long eofindex = -1;		/* When negative, putmail() can't truncate() */
+int dirhashes = 0;
 
 extern RETSIGTYPE wantout();
 extern time_t time();
@@ -295,7 +297,7 @@ extern int writebuf __((FILE *fp, char *buf, int len, int *lastchp));
 extern int writemimeline __((FILE *fp, char *buf, int len, int *lastchp));
 
 static int do_return_receipt_to = 0;
-static void  return_receipt __((struct ctldesc *dp, char *retrecpaddr));
+static void  return_receipt __((struct ctldesc *dp, char *retrecpaddr, char *uidstr));
 static char *find_return_receipt_hdr __((struct rcpt *rp));
 
 extern int  qp_to_8bit   __((struct rcpt *rp));
@@ -384,11 +386,16 @@ main(argc, argv)
 	errflg = 0;
 	logfile = NULL;
 	channel = CHANNEL;
-	while ((c = getopt(argc, argv, "abc:gh:Hl:rRSMV8")) != EOF) {
+	while ((c = getopt(argc, argv, "abc:Dgh:Hl:rRSMV8")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
 	    break;
+	  case 'D':
+	    ++dirhashes;	/* For user "abcdefg" the mailbox path is:
+				   MAILBOX/a/b/abcdefg -- supported by
+				   qpopper, for example... */
+	    break;
 	  case 'M':
 	    mmdf_mode = 1;
 	    break;
@@ -433,7 +440,7 @@ main(argc, argv)
 	  }
 	}
 	if (errflg || optind != argc) {
-	  fprintf(stderr, "Usage: %s [-V8abr] [-c channel] [-h host]\n",
+	  fprintf(stderr, "Usage: %s [-V8abDr] [-c channel] [-h host]\n",
 		  argv[0]);
 	  exit(EX_USAGE);
 	}
@@ -587,7 +594,7 @@ process(dp)
 	ts = ctime(&curtime);
 
 	if (!keep_header8 && headers_need_mime2(dp->recipients)) {
-	  headers_to_mime2(dp->recipients,verboselog);
+	  headers_to_mime2(dp->recipients,DefCharset,verboselog);
 	}
 
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
@@ -882,6 +889,9 @@ deliver(dp, rp, timestring)
 	      setrootuid(rp);
 	      return;
 	    }
+#ifdef HAVE_FSYNC
+	    fsync(fdmail);
+#endif
 	    close(fdmail);
 	  }
 	  setrootuid(rp);
@@ -1199,12 +1209,16 @@ deliver(dp, rp, timestring)
 #endif	/* BIFF || RBIFF */
 
 	fp = putmail(dp, rp, fdmail, "a+", timestring, file);
-	if (!fp)
-	  close(fdmail); /* Close now, as putmail() didn't
-			    do it at error return */
 	
 	if (S_ISREG(st.st_mode)) {
 
+	  if (fp) {
+	    fflush(fp);
+#ifdef HAVE_FSYNC
+	    fsync(fdmail);
+#endif
+	  }
+
 	  /* Previously acquired locks need to be released,
 	     preferrably in reverse order */
 
@@ -1262,18 +1276,20 @@ deliver(dp, rp, timestring)
 		       "2.2.0 (Delivered successfully)",
 		       "x-local; 250 (Delivered successfully)");
 	  DIAGNOSTIC(rp, EX_OK, "Ok", 0);
-	}
+	} else
+	  close(fdmail);
 #if	defined(HAVE_SOCKET)
-	else
+	if (fp) {
 	  if (nbp != NULL) /* putmail() has produced a DIAGNOSTIC */
 	    nbp->offset = -1;
+	}
 #endif	/* BIFF || RBIFF */
 
 	/* [Thomas Knott]  "Return-Receipt-To:" */
 	if (do_return_receipt_to) {
 	  retrecptaddr = find_return_receipt_hdr(rp);
 	  if (retrecptaddr != NULL)
-	    return_receipt(dp,retrecptaddr);
+	    return_receipt(dp,retrecptaddr,rp->addr->misc);
 	}
 
 	return;
@@ -1329,7 +1345,6 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  eofindex = 0;
 	}
 
-
 	if (!(convert_qp && qp_to_8bit(rp)))
 	  convert_qp = 0; /* If malloc failed, clear this too,
 			     if it was not set, it doesn't change.. */
@@ -1347,10 +1362,20 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  char **hdrs = has_header(rp,"Return-Path:");
 	  if (hdrs) delete_header(rp,hdrs);
 	  append_header(rp,"Return-Path: <%s>\n", fromuser);
+
+	  hdrs = has_header(rp,"To:");
+	  if (!hdrs) {
+	    /* No "To:" -header ?  Rewrite possible "Apparently-To:" header! */
+	    /* Sendmailism... */
+	    hdrs = has_header(rp,"Apparently-To:");
+	    if (hdrs) delete_header(rp,hdrs);
+	    append_header(rp,"Apparently-To: <%s>\n", rp->addr->link->user);
+	  }
 	}
+	
 
 	if (*fromuser == 0)
-	  fromuser = "postmaster";
+	  fromuser = "MAILER-DAEMON";
 
 	if (fprintf(fp, "%s%s %s", FROM_, fromuser, timestring) < 0
 	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0) {
@@ -1364,6 +1389,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  fflush(fp);
 	  if (eofindex >= 0)
 	    ftruncate(FILENO(fp), (u_long)eofindex);
+#endif
+#ifdef HAVE_FSYNC
+	  fsync(FILENO(fp));
 #endif
 	  fclose(fp);
 	  fp = NULL;
@@ -1384,6 +1412,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  fflush(fp);
 	  if (eofindex >= 0)
 	    ftruncate(FILENO(fp), (off_t)eofindex);
+#endif
+#ifdef HAVE_FSYNC
+	  fsync(FILENO(fp));
 #endif
 	  fclose(fp);
 	  eofindex = -1;
@@ -1421,6 +1452,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    if (eofindex >= 0)
 	      ftruncate(FILENO(fp), (off_t)eofindex);
 #endif /* HAVE_FTRUNCATE */
+#ifdef HAVE_FSYNC
+	    fsync(FILENO(fp));
+#endif
 	    fclose(fp);
 	    fp = NULL;
 	    goto time_reset;
@@ -1670,6 +1704,7 @@ creatembox(rp, filep, uid, gid, pw)
 {
 	char **maild;
 	int fd = -1;
+	char *s;
 
 	*uid = pw->pw_uid;
 	*gid = pw->pw_gid;
@@ -1677,8 +1712,27 @@ creatembox(rp, filep, uid, gid, pw)
 	for (maild = &maildirs[0]; *maild != 0; maild++) {
 	  if (*filep != NULL)
 	    free(*filep);
-	  *filep = emalloc(2+strlen(*maild)+strlen(rp->addr->user));
-	  sprintf(*filep, "%s/%s", *maild, rp->addr->user);
+	  *filep = emalloc(8+2+strlen(*maild)+strlen(rp->addr->user));
+	  sprintf(*filep, "%s/", *maild);
+	  s = *filep + strlen(*filep);
+	  if (dirhashes) {
+	    switch (dirhashes) {
+	    case 1:
+	      sprintf(s,"%c/",rp->addr->user[0]);
+	      s += 2;
+	      break;
+	    case 2:
+	      if (rp->addr->user[1])
+		sprintf(s,"%c/%c/",rp->addr->user[0],rp->addr->user[1]);
+	      else /* Err.... One char userid ?? TROUBLE TIME! */
+		sprintf(s,"%c/%c/",rp->addr->user[0],rp->addr->user[0]);
+	      s += 4;
+	      break;
+	    default:
+	      break;
+	    }
+	  }
+	  strcat(s, rp->addr->user);
 	  if ((fd = createfile(rp, *filep, *uid)) >= 0) {
 #ifdef	HAVE_FCHOWN
 	    fchown(fd, *uid, *gid);
@@ -2447,9 +2501,27 @@ char *str;
 
 
 static void
-return_receipt (dp, retrecptaddr)
+decodeXtext(fp,xtext)
+	FILE *fp;
+	char *xtext;
+{
+	for (;*xtext;++xtext) {
+	  if (*xtext == '+') {
+	    int c = '?';
+	    sscanf(xtext+1,"%02X",&c);
+	    putc(c,fp);
+	    if (*xtext) ++xtext;
+	    if (*xtext) ++xtext;
+	  } else
+	    putc(*xtext,fp);
+	}
+}
+
+static void
+return_receipt (dp, retrecptaddr, uidstr)
 	struct ctldesc *dp;
 	char *retrecptaddr;
+	char *uidstr;
 {
 	char buf[BUFSIZ];
 	char **cpp, *mailshare, *mfpath;
@@ -2458,17 +2530,26 @@ return_receipt (dp, retrecptaddr)
 	int n;
 	char lastchar;
 	char *boundarystr = buildboundary("return-receipt");
+	struct passwd *pw;
+	int uid;
+	struct stat stb;
+	char *username = "unknown";
 
 	char *dfltform[] = {
-		"From: The Post Office <postmaster>",
 		"Subject: Returned mail: Return receipt",
 		"MIME-Version: 1.0",
+		"Priority: junk",
 		"Content-Type: multipart/report; report-type=delivery-status;",
 		"",
 		"Your mail message has been delivered properly to the following recipients:",
 		NULL
 	};
 
+	uid = atoi(uidstr);
+	pw = getpwuid(uid);
+	if (pw)
+	  username = pw->pw_name;
+
 	if ((mfp = mail_open(MSG_RFC822)) == NULL) {
 	  for (rp = dp->recipients; rp != NULL; rp = rp->next)
 	    DIAGNOSTIC(rp, EX_TEMPFAIL, "mail_open failure", 0);
@@ -2482,9 +2563,11 @@ return_receipt (dp, retrecptaddr)
 	/* copy To: from return-receipt address */
 	fprintf(mfp, "todsn NOTIFY=NEVER ORCPT=rfc822;");
 	encodeXtext(mfp, retrecptaddr);
-	fprintf(mfp, "\nto <%s>\n",retrecptaddr);
+	fprintf(mfp, "\nto %s\n",retrecptaddr);
 	fprintf(mfp, "env-end\n");
 	fprintf(mfp, "To: %s\n", retrecptaddr);
+	fprintf(mfp, "From: Automatically on behalf of the user <%s>\n",
+		     username);
 
 	/* copy error message file itself */
 	if ((mailshare = getzenv("MAILSHARE")) == NULL)
@@ -2494,6 +2577,7 @@ return_receipt (dp, retrecptaddr)
 				 +strlen(RETURN_RECEIPT_FORM));
 	sprintf(mfpath, "%s/%s/%s", mailshare, FORMSDIR, RETURN_RECEIPT_FORM);
 
+
 	if ((efp = fopen(mfpath, "r")) != NULL) {
 	  int inhdr = 1;
 	  buf[sizeof(buf)-1] = 0;
@@ -2531,27 +2615,37 @@ return_receipt (dp, retrecptaddr)
 	fprintf(mfp, "\n--%s\n", boundarystr);
 	fprintf(mfp, "Content-Type: message/delivery-status\n\n");
 
-	/* XX: print out errors in IETF-NOTARY format as well! */
-	fprintf(mfp, "Original-MTS-Type: INET\n");
-	if (dp->envid != NULL)
-	  fprintf(mfp, "Original-Envelope-Id: %s\n",dp->envid);
-	fprintf(mfp, "Final-MTS-Type: INET\n");
-	fprintf(mfp, "Final-MTA: %s\n\n", mydomain() );
+	if (mydomain() != NULL) {
+	  fprintf(mfp, "Reporting-MTA: dns;%s\n", mydomain() );
+	} else {
+	  fprintf(mfp, "Reporting-MTA: x-local-hostname; -unknown-\n");
+	}
+	if (dp->envid != NULL) {
+	  fprintf(mfp, "Original-Envelope-Id: ");
+	  decodeXtext(mfp,dp->envid);
+	  putc('\n',mfp);
+	}
+	/* rfc822date() returns a string with trailing newline! */
+	fstat(dp->msgfd,&stb);
+	fprintf(mfp, "Arrival-Date: %s", rfc822date(&stb.st_mtime));
+	fprintf(mfp, "\n");
 
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
-	  fprintf(mfp, "Rcpt: %s\n", rp->addr->user);
+	  if (rp->orcpt != NULL) {
+	    fprintf(mfp, "Original-Recipient: ");
+	    decodeXtext(mfp,rp->orcpt);
+	    fprintf(mfp, "\n");
+	  }
+	  fprintf(mfp, "Final-Recipient: X-LOCAL;%s\n", rp->addr->user);
 	  fprintf(mfp, "Action: delivered\n");
-	  fprintf(mfp, "Status: 250 (%s)\n", rp->addr->host );
-	  if (rp->orcpt != NULL)
-	    fprintf(mfp, "Original-Rcpt: %s\n",rp->orcpt);
+	  fprintf(mfp, "Status: 2.5.0\n");
+	  fprintf(mfp, "Diagnostic-Code: smtp;250 (%s delivered)\n", rp->addr->host );
 	  fprintf(mfp, "\n");
 	}
 
 	fprintf(mfp, "--%s\n", boundarystr);
 	fprintf(mfp, "Content-Type: message/rfc822\n\n");
 
-	/* XX: Skip over the delivery envelope lines! */
-
 	rp = dp->recipients;
 	/* write the (new) headers with local "Received:"-line.. */
 	writeheaders(rp,mfp,"\n",0,0);
