@@ -4,6 +4,8 @@
  *	Copyright 1994 by Matti Aarnio -- MIME processings
  */
 
+#define DefCharset "UNKNOWN-8BIT"
+
 #include "hostenv.h"
 #include <stdio.h>
 #include <ctype.h>
@@ -115,22 +117,24 @@ struct exmapinfo {
 	int	origstatus;
 	char	*statusmsg;
 	int	newstatus;
+	char	*dsnstatus;
+	char	*dsndiags;
 } exmap[] = {
-{	EX_USAGE,	"command line usage error",	EX_TEMPFAIL	},
-{	EX_DATAERR,	"data format error",		EX_DATAERR	},
-{	EX_NOINPUT,	"cannot open input",		EX_TEMPFAIL	},
-{	EX_NOUSER,	"addressee unknown",		EX_NOUSER	},
-{	EX_NOHOST,	"host name unknown",		EX_NOHOST	},
-{	EX_UNAVAILABLE,	"service unavailable",		EX_UNAVAILABLE	},
-{	EX_SOFTWARE,	"internal software error",	EX_TEMPFAIL	},
-{	EX_OSERR,	"system error",			EX_TEMPFAIL	},
-{	EX_OSFILE,	"critical OS file missing",	EX_TEMPFAIL	},
-{	EX_CANTCREAT,	"can't create output file",	EX_TEMPFAIL	},
-{	EX_IOERR,	"input/output error",		EX_TEMPFAIL	},
-{	EX_TEMPFAIL,	"temporary failure",		EX_TEMPFAIL	},
-{	EX_PROTOCOL,	"remote error in protocol",	EX_TEMPFAIL	},
-{	EX_NOPERM,	"permission denied",		EX_NOPERM	},
-{	0,		NULL,				EX_TEMPFAIL	}
+  { EX_USAGE,	"command line usage error",	EX_TEMPFAIL,	"5.3.0", "x-local; 500 (Command line usage error)"	},
+  { EX_DATAERR,	"data format error",		EX_DATAERR,	"5.3.0", "x-local; 500 (Data format error)"	},
+  { EX_NOINPUT,	"cannot open input",		EX_TEMPFAIL,	"5.3.0", "x-local; 530 (Cannot open input)"	},
+  { EX_NOUSER,	"addressee unknown",		EX_NOUSER,	"5.1.1", "x-local; 521 (No such target user)"	},
+  { EX_NOHOST,	"host name unknown",		EX_NOHOST,	"5.3.0", "x-local; 500 (Target host unknown)"	},
+  { EX_UNAVAILABLE, "service unavailable",	EX_UNAVAILABLE,	"5.3.0", "x-local; 500 (Service unavailable)"	},
+  { EX_SOFTWARE, "internal software error",	EX_TEMPFAIL,	"5.3.0", "x-local; 500 (Internal software error)"	},
+  { EX_OSERR,	"system error",			EX_TEMPFAIL,	"5.3.0", "x-local; 500 (System error)"	},
+  { EX_OSFILE,	"critical OS file missing",	EX_TEMPFAIL,	"5.3.0", "x-local; 500 (Critical OS file missing)"	},
+  { EX_CANTCREAT, "can't create output file",	EX_TEMPFAIL,	"5.2.1", "x-local; 500 (Can't create output file)"	},
+  { EX_IOERR,	"input/output error",		EX_TEMPFAIL,	"5.2.2", "x-local; 500 (Input/Output error)"	},
+  { EX_TEMPFAIL, "temporary failure",		EX_TEMPFAIL,	"5.3.0", "x-local; 500 (Temporary failure)"	},
+  { EX_PROTOCOL, "remote error in protocol",	EX_TEMPFAIL,	"5.3.0", "x-local; 500 (Remote error in protocol)"	},
+  { EX_NOPERM,	"permission denied",		EX_NOPERM,	"5.2.0", "x-local; 520 (Permission denied)"	},
+  { 0,		NULL,				EX_TEMPFAIL,	NULL,	NULL	}
 };
 
 char myhostname[MAXHOSTNAMELEN+1];
@@ -316,6 +320,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 {
 	struct rcpt *rp;
 	struct exmapinfo *exp;
+	char *exs, *exd;
 	int i, j, pid, in[2], out[2];
 	unsigned int avsize;
 	FILE *tafp, *errfp;
@@ -333,10 +338,16 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	av[i++] = mp->argv[0];
 	if (mp->flags & MO_FFROMFLAG) {
 	  av[i++] = "-f";
-	  av[i++] = startrp->addr->link->user;
+	  if (strcmp(startrp->addr->link->channel,"error")==0)
+	    av[i++] = "<>";
+	  else
+	    av[i++] = startrp->addr->link->user;
 	} else if (mp->flags & MO_RFROMFLAG) {
 	  av[i++] = "-r";
-	  av[i++] = startrp->addr->link->user;
+	  if (strcmp(startrp->addr->link->channel,"error")==0)
+	    av[i++] = "<>";
+	  else
+	    av[i++] = startrp->addr->link->user;
 	}
 	for (j = 1; mp->argv[j] != NULL; ++j) {
 	  while (i+2 >= avsize) {
@@ -406,8 +417,8 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	if (pipe(out) < 0) {
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,"failed",
-			 "4.3.0 (Out of system resources, pipe creation failed)",
-			 "x-local; 400 (pipe creation error, out of system resources ?)");
+			 "5.3.0 (Out of system resources, pipe creation failed)",
+			 "x-local; 500 (pipe creation error, out of system resources ?)");
 	    diagnostic(rp, EX_OSERR, 0,
 		       "cannot create pipe from \"%s\"",
 		       mp->command);
@@ -417,8 +428,8 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	if (pipe(in) < 0) {
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,"failed",
-			 "4.3.0 (Out of system resources, pipe creation failed)",
-			 "x-local; 400 (pipe creation error, out of system resources ?)");
+			 "5.3.0 (Out of system resources, pipe creation failed)",
+			 "x-local; 500 (pipe creation error, out of system resources ?)");
 	    diagnostic(rp, EX_OSERR, 0,
 		       "cannot create pipe to \"%s\"",
 		       mp->command);
@@ -455,8 +466,8 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	} else if (pid < 0) {	/* couldn't fork, complain */
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,"failed",
-			 "4.3.0 (Out of system resources, fork failed)",
-			 "x-local; 400 (fork failure, out of system resources ?)");
+			 "5.3.0 (Out of system resources, fork failed)",
+			 "x-local; 500 (fork failure, out of system resources ?)");
 	    diagnostic(rp, EX_OSERR, 0, "cannot fork");
 	  }
 	  return;
@@ -488,7 +499,10 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  if (mp->flags & MO_CRLF) putc('\r',tafp);
 	  putc('\n',tafp);
 
-	  fprintf(tafp,"MAIL FROM:<%s>", startrp->addr->link->user);
+	  if (strcmp(startrp->addr->link->channel,"error")==0)
+	    fprintf(tafp,"MAIL From:<>");
+	  else
+	    fprintf(tafp,"MAIL From:<%s>", startrp->addr->link->user);
 	  if (mp->flags & MO_BESMTP) {
 	      fprintf(tafp," SIZE=%ld",startrp->desc->msgsizeestimate);
 	    if (can_8bit)
@@ -525,17 +539,18 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	if (mp->flags & (MO_UNIXFROM|MO_REMOTEFROM)) {
 	  char *timestring;
 	  time_t now;
+	  char *uu = startrp->addr->link->user;
 
 	  now = time((time_t *)0);
 	  timestring = ctime(&now);
 	  *(timestring+strlen(timestring)-1) = '\0';
-	  fprintf(tafp, "%s%s %s", FROM_,
-		  startrp->addr->link->user, timestring);
+	  if (strcmp(startrp->addr->link->channel,"error")==0)
+	    uu = "<>";
+	  fprintf(tafp, "%s%s %s", FROM_, uu, timestring);
 	  if (mp->flags & MO_REMOTEFROM)
 	    fprintf(tafp, " remote from %s", uucpname);
 	  if (verboselog) {
-	    fprintf(verboselog, "%s%s %s", FROM_,
-		    startrp->addr->link->user, timestring);
+	    fprintf(verboselog, "%s%s %s", FROM_, uu, timestring);
 	    if (mp->flags & MO_REMOTEFROM)
 	      fprintf(verboselog, " remote from %s", uucpname);
 	    putc('\n',verboselog);
@@ -591,7 +606,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  }
 	  
 	  if (!keep_header8 && headers_need_mime2(startrp)) {
-	    headers_to_mime2(startrp,verboselog);
+	    headers_to_mime2(startrp,DefCharset,verboselog);
 	  }
 	}
 
@@ -599,9 +614,11 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	   exist.. */
 	if (mp->flags & MO_RETURNPATH) {
 	  char **hdrs = has_header(startrp,"Return-Path:");
+	  char *uu = startrp->addr->link->user;
 	  if (hdrs) delete_header(startrp, hdrs);
-	  append_header(startrp,"Return-Path: <%s>\n",
-			startrp->addr->link->user);
+	  if (strcmp(startrp->addr->link->channel,"error")==0)
+	    uu = "";
+	  append_header(startrp,"Return-Path: <%s>\n", uu);
 	}
 	if (mp->flags & MO_CRLF)
 	  writeheaders(startrp, tafp, "\r\n", convertmode, maxwidth);
@@ -616,8 +633,8 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
 	    notaryreport(rp->addr->user,"failed",
 			 /* Could indicate: 4.3.1 - mail system full ?? */
-			 "4.3.0 (Write to target failed for some reason)",
-			 "x-local; 400 (Write to target failed for some reason)");
+			 "5.3.0 (Write to target failed for some reason)",
+			 "x-local; 500 (Write to target failed for some reason)");
 	    diagnostic(rp, i, 0, "write error");
 	  }
 	  /* just to make sure nothing will get delivered */
@@ -645,6 +662,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	close(in[0]);	/* more paranoia */
 	cp = buf + strlen(buf);
 
+	exd = exs = NULL;
 	pid = wait(&status);
 	if (WSIGNALSTATUS(status) != 0) {
 	  if (cp != buf)
@@ -654,6 +672,8 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	    strcat(cp, " (Core dumped)");
 	  strcat(cp, "]");
 	  i = EX_TEMPFAIL;
+	  exd = "x-local; 500 (failed on signal)";
+	  exs = "5.3.0";
 	} else if (WEXITSTATUS(status) == 0
 #if EX_OK != 0
 		   || WEXITSTATUS(status) == EX_OK
@@ -667,6 +687,8 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	    if (exp->origstatus == i) {
 	      s = exp->statusmsg;
 	      i = exp->newstatus;
+	      exs = exp->dsnstatus;
+	      exd = exp->dsndiags;
 	      break;
 	    }
 	  sprintf(cp, "[exit status %d", WEXITSTATUS(status));
@@ -680,12 +702,9 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	for (rp = startrp; rp != endrp; rp = rp->next) {
 	  if (i == EX_OK)
 	    notaryreport(rp->addr->user, "relayed",
-			 "2.2.0",
-			 "x-local; 250 Delivered");
+			 "2.5.0", "smtp;250 (Delivered)");
 	  else
-	    notaryreport(rp->addr->user, "failed",
-			 "5.3.0",
-			 "x-local; 530 Unknown error");
+	    notaryreport(rp->addr->user, "failed", exs, exd);
 	  diagnostic(rp, i, 0, "%s", buf);
 	}
 	/* XX: still need to deal with MO_STRIPQUOTES */
