@@ -27,10 +27,8 @@
 
 #define PROG "makendbm"
 
-#include <errno.h>	/* This (usually) defines the following 3.. */
-     /* extern int errno;
-	extern char *sys_errlist[];
-	extern int sys_nerr; */
+#include <errno.h>
+extern int errno;	/* Not all systems declare it in <errno.h> ... */
 
 extern char *strchr();
 
@@ -64,8 +62,7 @@ int errno;
   fprintf(stderr,"  (BTREE appends .db, to the actual db file name..)\n");
 #endif
   fprintf(stderr," Error now: %s",err);
-  if (errno > 0 && errno < sys_nerr)
-    fprintf(stderr,", errno=%d (%s)",errno,sys_errlist[errno]);
+  fprintf(stderr,", errno=%d (%s)",errno,strerror(errno));
   fprintf(stderr,"\n");
   exit (1);
 }
@@ -177,7 +174,7 @@ char *argv[];
 
 	if (argc < 3) usage(argv[0],"too few arguments",0);
 	if (argc > 4) usage(argv[0],"too many arguments",0);
-	dbasename = argv[1];
+	dbasename = argv[2];
 
 	if (argc == 4) {
 	  if (strcmp(argv[3],"-")==0)
@@ -244,8 +241,10 @@ char *argv[];
 	  gdbm_close(gdbmfile);
 #endif
 #ifdef HAVE_DB_H
-	if (typ == 3)
+	if (typ == 3) {
+	  (dbfile->sync)(dbfile,0);
 	  (dbfile->close)(dbfile);
+	}
 #endif
 
 	return 0;
